#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::TestEPagesConfig
# state        public
#----------------------------------------------------------------------------------------
# description  test the behavior of the EPagesConfig package
#========================================================================================
package TestEPagesConfig;
use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::EPagesConfig qw ( SetAuthorKeys GetAuthorKeys );
use Test::More;

my $EPAGES_CONFIG_RETURN_VALUE;
sub Main {
    OverwriteEpagesConfigCallsForTests();
    Test_AuthorKeys();
    Test_GetAuthorKeysIfUndefined();
    done_testing();
}

sub Test_AuthorKeys {
    my $subName = (caller(0))[3];
    SetAuthorKeys('abcLive','defLive','abcTest','defTest');
    my $ExpectedResult = {
                'UserNameLive' => 'abcLive',
                'PasswordLive' => 'defLive',
                'UserNameTest' => 'abcTest',
                'PasswordTest' => 'defTest',
        };
    # this tests fails if there is a value in the epages.config -> EncryptedDevelopmentKeys remove
    is_deeply(GetAuthorKeys(), $ExpectedResult, "$subName - test if we can read encryped values from config file - hash with authkeys");
}

sub Test_GetAuthorKeysIfUndefined {
    my $subName = (caller(0))[3];
    $EPAGES_CONFIG_RETURN_VALUE = undef;
    is(GetAuthorKeys(),undef,"$subName - behavoir if there is now value in epages config file - undef");
}

sub OverwriteEpagesConfigCallsForTests {
    ## no critic ( ProtectPrivateSubs)
    * DE_EPAGES::DHLIntraship::API::EPagesConfig::_SetValueToEpagesConfig = sub {
        my ( $EncryptedDevelopmentKeys ) = @_;
        $EPAGES_CONFIG_RETURN_VALUE = $EncryptedDevelopmentKeys;
    };
    * DE_EPAGES::DHLIntraship::API::EPagesConfig::_GetValueFromEpagesConfig = sub {
        return $EPAGES_CONFIG_RETURN_VALUE;
    };
    ## use critic ( ProtectPrivateSubs)
}

RunScript( Sub => \&Main );