#========================================================================================
# package      DE_EPAGES::Klarna::t::ReserveAmount_External
# state        public
#----------------------------------------------------------------------------------------
# description  External integration tests for Klarna API ReserveAmount Account
#========================================================================================
package DE_EPAGES::Klarna::t::ReserveAmount_External;

use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Klarna::UI::BasketOrder;
use Test::More;
use DE_EPAGES::Test::Mock::EpagesObject;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            if($ENV{SKIP_EXTERNAL_TESTS} ) {
                plan skip_all => 'call Klarna Server (SKIP_EXTERNAL_TESTS == 1)';
            }
            # tests
            my $Shop = _createShop();

            my $hParams = {
                'BasketOrder'    => _createOrderObject($Shop),
                'PaymentMethod'  => _createPaymentMethodObject($Shop),
                'SecurityNumber' => '07071960',
                'RemoteAddr'     => '1.2.3.4',
                'Street'         => 'Hellersbergstrae',
                'HouseNo'        => '14',
                'HouseNoExt'     => '',
            };
            my $Result = DE_EPAGES::Klarna::UI::BasketOrder->ReserveAmount($hParams);
            is($Result->{StatusCode}, 'OK', 'Successfull operation. Expected value OK.');
            done_testing();
        }
    );
}

sub _createLineItemPayment {
    my $hPaymentTypeInfo = {
        'Alias' => 'KlarnaHirePurchase'
    };
    my $PaymentType = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Payment::API::Object::PaymentType',$hPaymentTypeInfo);

    my $hKlarnaPeriodicCostInfo = {
        'Alias' => '3113'
    };
    my $KlarnaPeriodicCost = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Payment::API::Object::PaymentType',$hKlarnaPeriodicCostInfo);

    my $hLineItemPayment = {
        'Alias' => 'LineItemPaymentKlarna_5280F36E-E99C-5A3E-2085-AC153201AF0F',
        'NameOrAlias' => 'Klarna account',
        'Quantity' => 1,
        'Discount' => 0,
        'TaxRate' => '0.19',
        'PaymentType' => $PaymentType,
        'instanceOf' => 'LineItemPaymentKlarna',
        'BasePrice' => undef,
        'Kr_CellPhone' => '01522113356',
        'SKU' => undef,
        'InternalMethod' => 2,
        'KlarnaPeriodicCost' => $KlarnaPeriodicCost,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Klarna::API::Object::LineItemPaymentKlarna',$hLineItemPayment);
}

sub _createLineItemProduct {

    my $Product = _createProduct();
    my $hLineItemProduct = {
        'Alias' => 'Product 1',
        'NameOrAlias' => 'Product 1',
        'Quantity' => 1,
        'Discount' => 0,
        'TaxRate' => '0.19',
        'instanceOf' => 'LineItemProduct',
        'BasePrice' => 1234.567,
        'LineItemPrice' => '12345',
        'Kr_CellPhone' => '01522113356',
        'SKU' => undef,
        'Product' => $Product,
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Klarna::API::Object::LineItemPaymentKlarna',$hLineItemProduct);
}

sub _createProduct {
    my $OrderUnit = _createOrderUnit();
    my $hProductData = {
        'OrderUnit' => $OrderUnit
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', $hProductData);
}

sub _createOrderUnit {
    my $hPrarams = {
        'Alias' => ''
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::UnitOfMeasurement', $hPrarams);
}
sub _createShop {
    my $hShopInfo = {
        'VATreleased' => 0,
        'IsInternalTestShop' => 1,
    };

    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop',$hShopInfo);

    return $Shop;
}

sub _createAddress {

    my $hCountryInfo = {
        'Code2' => 'DE',
        'Code3' => 'DEU',
        'CountryID' => '276',
        'English  ' => 'Germany',
        'IsEU ' => '1',
        'IsUsed' => '1',
        'Native' => 'Deutschland',
    };

    my $hBillingAddressInfo = {
       'Alias' => 'Address',
       'City' => 'Neuss',
       'Country' => $hCountryInfo,
       'CountryID' => '276',
       'FirstName' => ' Testperson-de',
       'LastName' => 'Approved ',
       'Street' => 'Hellersbergstrae 14',
       'Zipcode' => '41460',
       'AddressExtension' => undef,
       'Phone' => undef,
       'PhoneCell' => undef,
       'PhonePrivate' => undef,
       'PhoneBusiness' => undef,
       'EMail' => 'blablabla@test.com',
       'EMailPrivate' => undef,
       'EMailBusiness' => undef,
       'Company' => undef,
       'Gender' => 1,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address',$hBillingAddressInfo);
}

sub _createLineItemContainer {
    my ($Shop) = @_;

    my $LineItemPayment = _createLineItemPayment();
    my $LineItemProduct = _createLineItemProduct();

    my $hLineItemContainerInfo = {
        'Alias' => 'LineItemContainer',
        'LanguageID' => '2',
        'Payment' => $LineItemPayment,
        'LineItems' => [$LineItemPayment, $LineItemProduct],
        'TaxModel' => 1,
        'Shipping' => undef,
        'Discounts' => [],
        'SalesDiscounts' => [],
        'GrandTotal' => '9.995',
        'CurrencyID' => 'EUR',
        'PaymentDiscount' => undef,
        'TotalTax' => '24.41',
        'Site' => $Shop,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer',$hLineItemContainerInfo);
}

sub _createOrderObject {
    my ($Shop) = @_;

    my $LineItemContainer = _createLineItemContainer($Shop);
    my $BillingAddress = _createAddress();
    my $hAtributes = {
       'Alias' => '1004',
       'BillingAddress' => $BillingAddress,
       'LineItemContainer' => $LineItemContainer,
   };
   return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address',$hAtributes)
}

sub _createPaymentMethodObject {
    my ($Shop) = @_;

    my $hPaymentTypeInfo = {
        'Alias' => 'KlarnaHirePurchase',
        'IsInTestMode' => 1,
        'Kr_StoreID' => 301,
        'Kr_SecretKey' => 'NoKn3kdeMKhxVmP',
        'Site' => $Shop
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Payment::API::Object::PaymentType',$hPaymentTypeInfo);

}
RunScript( Sub => \&Main );

1;