#========================================================================================
# package      DE_EPAGES::Klarna::t::Test_UI_BasketOrder
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI BasketOrder
# state        public
#========================================================================================
package Test_UI_BasketOrder;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use Test::More;
use Test::Exception;
use DE_EPAGES::Klarna::UI::BasketOrder;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Klarna::API::Constants qw ( %PROTOCOL );

sub Main {
    Test_sendOrderToKlarna();
    Test_createOrder();
    Test_saveResponseToLineItemPayment();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub Test_sendOrderToKlarna{
    my $subTestName = (caller(0))[3];
    my $BasketOrderMock = _createBasketOrderObjectAndOverwriteBindings();

    $BasketOrderMock->{'IsAdvancedIntegration'} = 0;
    my $hParams = {
        'CallerName' => undef,
        'BasketOrder' => _createBasketOrOrder(),
    };
    $hParams->{'BasketOrder'}->get('LineItemContainer')->get('Payment')->set({'Protocol'=>$PROTOCOL{'Standard'}});
    my $hResult = $BasketOrderMock->_sendOrderToKlarna($hParams);## no critic ( ProhibitPrivateSubroutineUse )
    is($hResult->{'CallerName'},'AddTransaction',
        "$subTestName - call AddTransaction if this is the Standard integration - AddTransaction");

    $hParams->{'BasketOrder'}->get('LineItemContainer')->get('Payment')->set({'Protocol'=>$PROTOCOL{'Advanced'}});
    $BasketOrderMock->{'IsAdvancedIntegration'} = 1;
    $hResult = $BasketOrderMock->_sendOrderToKlarna($hParams);## no critic ( ProhibitPrivateSubroutineUse )
    is($hResult->{'CallerName'},'ReserveAmount',
        "$subTestName - call ReserveAmount if this is the advanced integration - ReserveAmount");
}

#----------------------------------------------------------------------------------------
sub Test_createOrder {
    ## no critic ( ProhibitPrivateSubroutineUse )
    my $subTestName = (caller(0))[3];
    my $BasketOrderMock = _createBasketOrderObjectAndOverwriteBindings();

    # test for Basket
    my $BasketOrOrder = _createBasketOrOrder('Basket');
    my $Basket = $BasketOrderMock->_createOrder($BasketOrOrder);
    is ($Basket->get('ControllValue'),'called in Basket2Order',"$subTestName - was a Basket - called in Basket2Order");

    # test Customer Order
    $BasketOrOrder = _createBasketOrOrder('CustomerOrder');
    $Basket = $BasketOrderMock->_createOrder($BasketOrOrder);
    is ($Basket->get('ControllValue'),'CustomerOrder',"$subTestName - was a CustomerOrder - CustomerOrder");

    # test for some errors
    throws_ok { $BasketOrderMock->_createOrder() } qr/Can't call method "instanceOf" on an undefined value at/
            , "$subTestName - expected Error if the Argument is undef";
    $BasketOrOrder = _createBasketOrOrder('WrongType');
    throws_ok { $BasketOrderMock->_createOrder($BasketOrOrder) } qr/UnknownInstanceType/
            , "$subTestName - expected Error if the Argument is unknown";
    ## use critic ( ProhibitPrivateSubroutineUse )
}

#----------------------------------------------------------------------------------------
sub Test_saveResponseToLineItemPayment {
    my $subTestName = (caller(0))[3];
    # prepare Standard Integration
    my $BasketOrderMock = _createBasketOrderObjectAndOverwriteBindings();
    my $LineItemPayment = _createLineItemPayment();
    $LineItemPayment->set({'Protocol'=>$PROTOCOL{'Standard'}});
    my $hFakeResponse = _createFakeResponse();
    #call
    $BasketOrderMock->_saveResponseToLineItemPayment($hFakeResponse, $LineItemPayment ); ## no critic ( ProhibitPrivateSubroutineUse )
    #tests
    is ($LineItemPayment->get('Kr_InvoiceNo'),'AValueFromResponse',
        "$subTestName - check if the Standard Intregration cointains the Value - AValueFromResponse");
    is ($LineItemPayment->{'Attributes'}->{'ReservationNumber'}, undef,
        "$subTestName - check if the advanced Intregration cointains not the Value - undef");
    is ($LineItemPayment->get('TransTime'), 'FakeTimeStamp',
        "$subTestName - is the transTime set? - FakeTimeStamp");
    is ($LineItemPayment->get('Message'), undef,
        "$subTestName - is message reset? - AValueFromResponse");

    # prepare advanced integration
    $LineItemPayment = _createLineItemPayment();
    $LineItemPayment->set({'Protocol'=>$PROTOCOL{'Advanced'}});
    $hFakeResponse = _createFakeResponse();
    #call
    $BasketOrderMock->_saveResponseToLineItemPayment($hFakeResponse, $LineItemPayment ); ## no critic ( ProhibitPrivateSubroutineUse )
    #tests

    is ($LineItemPayment->get('ReservationNumber'),'AValueFromResponse',
        "$subTestName - check if the advanced Intregration cointains the Value - AValueFromResponse");
    is ($LineItemPayment->{'Attributes'}->{'Kr_InvoiceNo'}, undef,
        "$subTestName - check if the Standard Intregration cointains not the Value - undef");
    is ($LineItemPayment->get('TransTime'), 'FakeTimeStamp',
        "$subTestName - is the transTime set? - FakeTimeStamp");
    is ($LineItemPayment->get('Message'), undef,
        "$subTestName - is message reset? - undef");
}

#----------------------------------------------------------------------------------------
sub _createFakeResponse {
    return {
        'Message' =>[
            {'string' => 'AValueFromResponse'}
        ]
    };
}

#----------------------------------------------------------------------------------------
sub _createLineItemPayment {
    my $hAttributes = {
        'Message' => 'some Message',
        'Payment' => _createPayment(),
        'Protocol' => undef,
        'ReservationNumber' => undef
    };
    my $NameSpace = 'DE_EPAGES::Klarna::API::Object::LineItemPaymentKlarna';
    return DE_EPAGES::Test::Mock::EpagesObject->new( $NameSpace, $hAttributes);
}

#----------------------------------------------------------------------------------------
sub _createBasketOrOrder {
    my ($InstanceName) = @_;
    my $hAttributes = {
        'instanceOf' => $InstanceName,
        'ControllValue' => 'CustomerOrder',
        'LineItemContainer' => _createLineItemPayment(),
    };
    my $NameSpace = 'DE_EPAGES::Order::API::Order';
    return DE_EPAGES::Test::Mock::EpagesObject->new( $NameSpace, $hAttributes);
}

#----------------------------------------------------------------------------------------
sub _createPayment {
    my $hAttributes = {
        'Message' => 'some Message',
    };
    my $NameSpace = 'DE_EPAGES::Klarna::API::Object::LineItemPaymentKlarna';
    return DE_EPAGES::Test::Mock::EpagesObject->new( $NameSpace, $hAttributes);
}

#----------------------------------------------------------------------------------------
sub _createBasketOrderObjectAndOverwriteBindings {
    my $BasketOrder = DE_EPAGES::Klarna::UI::BasketOrder->new();
    my $BasketOrderMock = Test::MockObject::Extends->new( $BasketOrder );
    $BasketOrderMock->mock('_GetCurrentDateTime',sub{return 'FakeTimeStamp';});
    $BasketOrderMock->mock('_Basket2Order'
                            ,sub{
                                    my $self = shift;
                                    my ($BasketOrOrder) = @_;
                                    $BasketOrOrder->set({'ControllValue' => 'called in Basket2Order'}); # wrong name
                                    return $BasketOrOrder;
                            }
                          );
    $BasketOrderMock->mock('ReserveAmount'
                            ,sub{
                                    my $self = shift;
                                    my ($hParams) = @_;
                                    $hParams->{'CallerName'} = 'ReserveAmount';
                                    return $hParams;
                            }
                          );
    $BasketOrderMock->mock('AddTransaction'
                            ,sub{
                                    my $self = shift;
                                    my ($hParams) = @_;
                                    $hParams->{'CallerName'} = 'AddTransaction';
                                    return $hParams;
                            }
                          );
    return $BasketOrderMock;
}

RunScript( Sub => \&Main );