#========================================================================================
# package      DE_EPAGES::Shipping::t::ShippingMethodResource
# state        private
# base         DE_EPAGES::Shop::API::Test
# description  test the REST resource for a single shipping method.
#========================================================================================
package DE_EPAGES::Shipping::t::ShippingMethodResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Data::Dumper;
use Test::More tests => 4;

use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Shipping::API::Resource::ShippingMethod;
use DE_EPAGES::Test::Mock::EpagesObject;

my $ShopDomain = 'test-domain';

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';

    note 'importing import.xml';
    ImportObjects('import.xml', $self->shop->parent->pathString);

    $self->shop->set({
        DomainName => $ShopDomain,
        HasSSLCertificate => 1,
    });
}

sub _shippingMethod {
    my $self = shift;
    my ($Alias) = @_;

    return $self->shop->child('ShippingMethods')->child($Alias);
}

sub _TestError(&$$) {
    my ( $Sub, $ExpectedCode, $Message ) = @_;

    eval { $Sub->() };
    ok ExistsError(), "$Message: error expected";
    my $Error = GetError();
    is $Error->code, $ExpectedCode, "$Message: error code";
}

sub test {
    my $self = shift;

    $self->testVisibleMethod;
    $self->testInVisibleMethod;
    $self->testNoLogo;
}

sub testVisibleMethod {
    my $self = shift;

    my $ShippingMethod = $self->_shippingMethod('TestShipping1');

    my $Resource = DE_EPAGES::Shipping::API::Resource::ShippingMethod->new;
    my $hData = $Resource->get($ShippingMethod, 'de_DE');

    is_deeply( $hData, {
        'logo' => "https://$ShopDomain/WebRoot/Store/Test/TestShop/ShippingLogos/logo1.gif",
        'shippingMethodId' => $ShippingMethod->guid,
        'name' => 'Kostenfreier Versand',
        'description' => 'Wirklich keine weiteren Kosten'
    }, 'get' ) or diag Dumper($hData);
}

sub testInVisibleMethod {
    my $self = shift;

    my $ShippingMethod = $self->_shippingMethod('TestShipping2');
    my $Resource = DE_EPAGES::Shipping::API::Resource::ShippingMethod->new;

    _TestError sub { $Resource->get($ShippingMethod, 'de_DE') }, 'ResourceNotFound', 'invisible shipping method';
}

sub testNoLogo {
    my $self = shift;

    my $ShippingMethod = $self->_shippingMethod('TestShipping3');
    my $Resource = DE_EPAGES::Shipping::API::Resource::ShippingMethod->new;
    my $hData = $Resource->get($ShippingMethod, 'de_DE');

    is_deeply( $hData, {
        'logo' => undef,
        'shippingMethodId' => $ShippingMethod->guid,
        'name' => 'TestShipping3',
        'description' => undef,
    }, 'no description and no logo' ) or diag Dumper($hData);
}

__PACKAGE__->runScript;
